@@ -15,15 +15,16 @@ __status__ = "Development"
 
 
 class DigitalEffectsTableTemplates(ABC):
-    def __init__(self, yvariable, yunit, output_vars):
-        """
-        Initialise DigitalEffectsTableTemplates
+    """
+    DigitalEffectsTableTemplates class allows to make templates for digital effects table creation for measurand
 
-        :param yvariable: name of the measurand variable
-        :type yvariable: string
-        :param yunit: unit of the measurand
-        :type yunit: string
-        """
+    :param yvariable: name of the measurand variable
+    :type yvariable: string
+    :param yunit: unit of the measurand
+    :type yunit: string
+    """
+
+    def __init__(self, yvariable, yunit, output_vars):
         if isinstance(yvariable, str):
             self.yvariable = [yvariable]
             self.yunit = [yunit]
@@ -38,6 +39,7 @@ class DigitalEffectsTableTemplates(ABC):
         dims,
         dim_sizes,
         str_corr_dims=[],
+        separate_corr_dims=False,
         str_repeat_noncorr_dims=[],
         store_unc_percent=False,
         repeat_dim_err_corrs=[],
@@ -57,12 +59,12 @@ class DigitalEffectsTableTemplates(ABC):
         template = {}
 
         for i in range(self.output_vars):
-            err_corr, custom_err_corr = self.set_errcorr_shape(
+            err_corr, custom_err_corr = self._set_errcorr_shape(
                 dims[i],
                 dim_sizes,
                 "err_corr_str_" + self.yvariable[i],
                 str_repeat_noncorr_dims=str_repeat_noncorr_dims,
-                str_corr_dims=str_corr_dims,
+                str_corr_dims=str_corr_dims[i],
                 repeat_dim_err_corr=repeat_dim_err_corrs,
             )
 
@@ -77,17 +79,17 @@ class DigitalEffectsTableTemplates(ABC):
                 "attributes": {
                     "units": self.yunit[i],
                     "unc_comps": [
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i],
                             "ran",
                             store_unc_percent=store_unc_percent,
                         ),
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i],
                             "sys",
                             store_unc_percent=store_unc_percent,
                         ),
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i],
                             "str",
                             store_unc_percent=store_unc_percent,
@@ -96,7 +98,7 @@ class DigitalEffectsTableTemplates(ABC):
                 },
             }
             template[
-                self.make_ucomp_name(
+                self._make_ucomp_name(
                     self.yvariable[i], "ran", store_unc_percent=store_unc_percent
                 )
             ] = {
@@ -106,12 +108,12 @@ class DigitalEffectsTableTemplates(ABC):
                     "units": units,
                     "err_corr": [
                         {"dim": dim, "form": "random", "params": [], "units": []}
-                        for dim in dims
+                        for dim in dims[i]
                     ],
                 },
             }
             template[
-                self.make_ucomp_name(
+                self._make_ucomp_name(
                     self.yvariable[i], "sys", store_unc_percent=store_unc_percent
                 )
             ] = {
@@ -121,12 +123,12 @@ class DigitalEffectsTableTemplates(ABC):
                     "units": units,
                     "err_corr": [
                         {"dim": dim, "form": "systematic", "params": [], "units": []}
-                        for dim in dims
+                        for dim in dims[i]
                     ],
                 },
             }
             template[
-                self.make_ucomp_name(
+                self._make_ucomp_name(
                     self.yvariable[i], "str", store_unc_percent=store_unc_percent
                 )
             ] = {
@@ -147,6 +149,7 @@ class DigitalEffectsTableTemplates(ABC):
         dims,
         dim_sizes,
         str_corr_dims=[],
+        separate_corr_dims=False,
         str_repeat_noncorr_dims=[],
         store_unc_percent=False,
         repeat_dim_err_corrs=[],
@@ -164,12 +167,12 @@ class DigitalEffectsTableTemplates(ABC):
         template = {}
 
         for i in range(self.output_vars):
-            err_corr, custom_err_corr = self.set_errcorr_shape(
+            err_corr, custom_err_corr = self._set_errcorr_shape(
                 dims[i],
                 dim_sizes,
                 "err_corr_tot_" + self.yvariable[i],
                 str_repeat_noncorr_dims=str_repeat_noncorr_dims,
-                str_corr_dims=str_corr_dims,
+                str_corr_dims=str_corr_dims[i],
                 repeat_dim_err_corr=repeat_dim_err_corrs,
             )
 
@@ -184,7 +187,7 @@ class DigitalEffectsTableTemplates(ABC):
                 "attributes": {
                     "units": self.yunit[i],
                     "unc_comps": [
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i],
                             "tot",
                             store_unc_percent=store_unc_percent,
@@ -193,7 +196,7 @@ class DigitalEffectsTableTemplates(ABC):
                 },
             }
             template[
-                self.make_ucomp_name(
+                self._make_ucomp_name(
                     self.yvariable[i], "tot", store_unc_percent=store_unc_percent
                 )
             ] = {
@@ -222,6 +225,7 @@ class DigitalEffectsTableTemplates(ABC):
         dims,
         dim_sizes,
         str_corr_dims=[],
+        separate_corr_dims=False,
         str_repeat_noncorr_dims=[],
         store_unc_percent=False,
         repeat_dim_err_corrs=[],
@@ -252,7 +256,7 @@ class DigitalEffectsTableTemplates(ABC):
                 "attributes": {
                     "units": self.yunit[i],
                     "unc_comps": [
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i], comp, store_unc_percent=store_unc_percent
                         )
                         for comp in comp_list
@@ -263,7 +267,7 @@ class DigitalEffectsTableTemplates(ABC):
             for ic, comp in enumerate(comp_list):
                 if comp == "random" and simple_random:
                     template[
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i], comp, store_unc_percent=store_unc_percent
                         )
                     ] = {
@@ -285,7 +289,7 @@ class DigitalEffectsTableTemplates(ABC):
 
                 elif comp == "systematic" and simple_systematic:
                     template[
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i], comp, store_unc_percent=store_unc_percent
                         )
                     ] = {
@@ -306,17 +310,17 @@ class DigitalEffectsTableTemplates(ABC):
                     }
 
                 else:
-                    err_corr, custom_err_corr = self.set_errcorr_shape(
+                    err_corr, custom_err_corr = self._set_errcorr_shape(
                         dims[i],
                         dim_sizes,
                         "err_corr_" + comp + "_" + self.yvariable[i],
                         str_repeat_noncorr_dims=str_repeat_noncorr_dims,
-                        str_corr_dims=str_corr_dims,
+                        str_corr_dims=str_corr_dims[i],
                         repeat_dim_err_corr=repeat_dim_err_corrs[ic],
                     )
 
                     template[
-                        self.make_ucomp_name(
+                        self._make_ucomp_name(
                             self.yvariable[i], comp, store_unc_percent=store_unc_percent
                         )
                     ] = {
@@ -331,14 +335,26 @@ class DigitalEffectsTableTemplates(ABC):
 
         return template
 
-    def make_ucomp_name(self, var, ucomp, store_unc_percent=False):
+    def _make_ucomp_name(self, var, ucomp, store_unc_percent=False):
+        """
+        Return the name of the uncertainty component
+
+        :param var: name of variable
+        :type var: str
+        :param ucomp: name of component
+        :type ucomp: str
+        :param store_unc_percent: boolean to indicate whether uncertainties are stored in percent, defaults to False.
+        :type store_unc_percent: bool, optional
+        :return: name of uncertainty component
+        :rtype: str
+        """
         if store_unc_percent:
             uvarname_start = "u_rel_"
         else:
             uvarname_start = "u_"
         return uvarname_start + ucomp + "_" + var
 
-    def set_errcorr_shape(
+    def _set_errcorr_shape(
         self,
         dims,
         dim_sizes,
@@ -349,6 +365,7 @@ class DigitalEffectsTableTemplates(ABC):
     ):
         """
         Function to work out which di
+
         :param dims: list of dimensions
         :type dims: list
         :param u_xvar_ref: reference uncertainty component that is used to populate repeated dims
@@ -371,7 +388,7 @@ class DigitalEffectsTableTemplates(ABC):
             else:
                 custom_corr_dims.append(dim)
 
-        if len(str_corr_dims) == 0:
+        if len(str_corr_dims) == 0 or all(elem is None for elem in str_corr_dims):
             if len(custom_corr_dims) > 0:
                 if len(custom_corr_dims) == 1:
                     corrdim = custom_corr_dims[0]
@@ -403,20 +420,21 @@ class DigitalEffectsTableTemplates(ABC):
         else:
             custom_err_corr_dict = {}
             for corrdim in str_corr_dims:
-                custom_err_corr_dict[err_corr_name + "_" + corrdim] = {
-                    "dtype": np.float32,
-                    "dim": [corrdim, corrdim],
-                    "attributes": {"units": ""},
-                }
-
-                err_corr_list.append(
-                    {
-                        "dim": corrdim.split("."),
-                        "form": "err_corr_matrix",
-                        "params": [err_corr_name + "_" + corrdim],
-                        "units": [],
+                if corrdim is not None:
+                    custom_err_corr_dict[err_corr_name + "_" + corrdim] = {
+                        "dtype": np.float32,
+                        "dim": [corrdim, corrdim],
+                        "attributes": {"units": ""},
                     }
-                )
+
+                    err_corr_list.append(
+                        {
+                            "dim": corrdim.split("."),
+                            "form": "err_corr_matrix",
+                            "params": [err_corr_name + "_" + corrdim],
+                            "units": [],
+                        }
+                    )
 
         return err_corr_list, custom_err_corr_dict
 
