@@ -12,6 +12,31 @@ __status__ = "Development"
 
 
 class MeasurementFunctionUtils:
+    """
+    Class with utility functions to help in the uncertainty propagation and handling of error correlations
+
+    :param xvariables: list of input quantity names, in same order as arguments in measurement function and with same exact names as provided in input datasets.
+    :type xvariables: list(str)
+    :param uncxvariables: list of input quantity names for which uncertainties should be propagated. Should be a subset of input quantity names. Defaults to None, in which case uncertainties on all input quantities are used.
+    :type uncxvariables: list(str), optional
+    :param ydims: list of dimensions of the measurand, in correct order. list of list of dimensions when there are multiple measurands. Default to None, in which case it is assumed to be the same as refxvar (see below) input quantity.
+    :type ydims: list(str), optional
+    :param str_repeat_dims: Strings used to select the axis which has repeated measurements. Axis can be specified using the name(s) of the dimension.
+    :type str_repeat_dims: list(str)
+    :param str_repeat_noncorr_dims: list of dimension names to be used as repeated dims
+    :type str_repeat_noncorr_dims: list(str)
+    :param verbose: boolean to set verbosity
+    :type verbose: bool
+    :param templ: templater object
+    :type templ: punpy.digital_effects_table_template
+    :param use_err_corr_dict: when possible, use dictionaries with separate error-correlation info per dimension in order to save memory
+    :type use_err_corr_dict: bool, optional
+    :param broadcast_correlation: correlation form ("rand" or "syst" to use when broadcasting
+    :type broadcast_correlation: str
+    :param param_fixed: set to true or false to indicate for each input quantity whether it has to remain unmodified either when expand=true or when using repeated measurements, defaults to None (no inputs fixed).
+    :type param_fixed: list of bools, optional
+    """
+
     def __init__(
         self,
         xvariables,
@@ -25,20 +50,6 @@ class MeasurementFunctionUtils:
         broadcast_correlation,
         param_fixed,
     ):
-        """
-        Initialise MeasurementFunctionUtils
-
-        :param xvariables: list of input quantity names, in same order as arguments in measurement function and with same exact names as provided in input datasets.
-        :type xvariables: list(str)
-        :param uncxvariables: list of input quantity names for which uncertainties should be propagated. Should be a subset of input quantity names. Defaults to None, in which case uncertainties on all input quantities are used.
-        :type uncxvariables: list(str), optional
-        :param str_repeat_noncorr_dims: list of dimension names to be used as repeated dims
-        :type str_repeat_noncorr_dims: list(str)
-        :param verbose: boolean to set verbosity
-        :type verbose: bool
-        :param templ: templater object
-        :type templ: punpy.digital_effects_table_template
-        """
         self.xvariables = xvariables
         self.uncxvariables = uncxvariables
         self.verbose = verbose
@@ -51,7 +62,7 @@ class MeasurementFunctionUtils:
         self.broadcast_correlation = broadcast_correlation
         self.param_fixed = param_fixed
 
-    def find_repeat_dim_corr(self, form, *args, store_unc_percent=False, ydims=None):
+    def _find_repeat_dim_corr(self, form, *args, store_unc_percent=False, ydims=None):
         """
         Function to make dict with the error correlation information along the repeated dims.
 
@@ -90,7 +101,7 @@ class MeasurementFunctionUtils:
                                     continue
                                 elif repeat_dim in dataset[var].dims:
                                     repeat_dims_errcorrs = (
-                                        self.set_repeat_dims_errcorrs(
+                                        self._set_repeat_dims_errcorrs(
                                             comps,
                                             dataset,
                                             var,
@@ -99,7 +110,7 @@ class MeasurementFunctionUtils:
                                         )
                                     )
                                 else:
-                                    self.check_repeat_err_corr_same(
+                                    self._check_repeat_err_corr_same(
                                         repeat_dims_errcorrs[repeat_dim], "systematic"
                                     )
                                     repeat_dims_errcorrs[repeat_dim][
@@ -107,13 +118,28 @@ class MeasurementFunctionUtils:
                                     ] = "systematic"
         return repeat_dims_errcorrs
 
-    def set_repeat_dims_errcorrs(
+    def _set_repeat_dims_errcorrs(
         self, comps, dataset, var, repeat_dim, repeat_dims_errcorrs
     ):
+        """
+        Set repeat_dims_errcorr dictionary values with the error correlation information along the repeated dims.
+
+        :param comps: name of uncertainty component
+        :type comps: str
+        :param dataset: dataset being querried
+        :type dataset: xarray.dataset
+        :param var: given variable
+        :type var: str
+        :param repeat_dims: Used to select the axis which has repeated measurements using the name(s) of the dimension.
+        :type repeat_dims: str
+        :param repeat_dims_errcorr: dictionary with errcorr info for repeated dims
+        :type repeat_dims_errcorr: dict
+        :return:
+        """
         for comp in comps:
             for idim in range(len(dataset[var].dims)):
                 if dataset[comp].attrs["err_corr_%s_dim" % (idim + 1)] == repeat_dim:
-                    self.check_repeat_err_corr_same(
+                    self._check_repeat_err_corr_same(
                         repeat_dims_errcorrs[repeat_dim],
                         dataset[comp],
                         idim,
@@ -130,7 +156,7 @@ class MeasurementFunctionUtils:
 
         return repeat_dims_errcorrs
 
-    def check_repeat_err_corr_same(self, repeat_dims_errcorr, uvar, idim=None):
+    def _check_repeat_err_corr_same(self, repeat_dims_errcorr, uvar, idim=None):
         """
         Function to check whether the err_corr along the repeated dim for specific uncertainty variable is the same as the current errcorr dictionary.
 
@@ -176,7 +202,7 @@ class MeasurementFunctionUtils:
                     % (repeat_dims_errcorr["dim"])
                 )
 
-    def set_repeat_dims_form(self, repeat_dims_errcorrs):
+    def _set_repeat_dims_form(self, repeat_dims_errcorrs):
         """
         Function to set self.repeat_dims_form to the correct form.
 
@@ -271,7 +297,7 @@ class MeasurementFunctionUtils:
                     comps.append(*list(comps_rand.variables.mapping.keys()))
 
         else:
-            compname = self.templ.make_ucomp_name(
+            compname = self.templ._make_ucomp_name(
                 var, form, store_unc_percent=store_unc_percent
             )
             if compname in comps:
@@ -359,6 +385,8 @@ class MeasurementFunctionUtils:
         :type sizes_dict: dict
         :param expand: boolean to indicate whether the input quantities should be expanded/broadcasted to the shape of the measurand.
         :type expand: bool
+        :param corr_dims: set to positive integer to select the axis used in the correlation matrix. The correlation matrix will then be averaged over other dimensions. Defaults to -99, for which the input array will be flattened and the full correlation matrix calculated.
+        :type corr_dims: integer, optional
         :return: list of uncertainty values (as np.ndarray) for each of the input quantites.
         :rtype: list(np.ndarray)
         """
@@ -376,14 +404,9 @@ class MeasurementFunctionUtils:
                     if hasattr(dataset, "variables"):
                         if var in dataset.variables:
                             if ydims[0] is not None:
-                                if np.all(
+                                if expand and not np.all(
                                     [dim in dataset[var].dims for dim in required_dims]
                                 ):
-                                    inputs_unc[iv] = self.calculate_unc(
-                                        form, dataset, var
-                                    )
-
-                                else:
                                     inputs_unc[iv] = self.calculate_unc_missingdim(
                                         form,
                                         dataset,
@@ -392,6 +415,10 @@ class MeasurementFunctionUtils:
                                         sizes_dict=sizes_dict,
                                         ydims=ydims,
                                     )
+                                else:
+                                    inputs_unc[iv] = self.calculate_unc(
+                                        form, dataset, var
+                                    )
 
                 if np.count_nonzero(inputs_unc[iv]) == 0:
                     inputs_unc[iv] = None
@@ -442,7 +469,7 @@ class MeasurementFunctionUtils:
                 uvar = "%s_%s" % (form, var)
                 data = ds[uvar]
                 if data.attrs["units"] == "%":
-                    return data.values / 100 * ds[var].values
+                    return data.values / 100 * np.abs(ds[var].values)
             except:
                 keys = np.array(list(ds.keys()))
                 uvar_ids = [
@@ -453,7 +480,7 @@ class MeasurementFunctionUtils:
                 if len(uvar) > 0:
                     data = ds[uvar[0]]
                     if data.attrs["units"] == "%":
-                        return data.values / 100 * ds[var].values
+                        return data.values / 100 * np.abs(ds[var].values)
                     else:
                         return data.values
 
@@ -504,7 +531,11 @@ class MeasurementFunctionUtils:
             else:
                 usyst = None
 
-            out = [ucomp ** 2 for ucomp in [ustru, urand, usyst] if ucomp is not None]
+            out = [
+                ucomp**2
+                for ucomp in [ustru, urand, usyst]
+                if ucomp is not None and len(ucomp)
+            ]
 
             out = np.sum(out, axis=0) ** 0.5
         else:
@@ -553,7 +584,7 @@ class MeasurementFunctionUtils:
                                     var,
                                     expand=expand,
                                     sizes_dict=sizes_dict,
-                                    ydims=ydims,
+                                    ydims=ydims[0],
                                 )
                             else:
                                 inputs_corr[iv] = self.calculate_corr(
@@ -595,7 +626,7 @@ class MeasurementFunctionUtils:
         if form == "tot":
             if self.use_err_corr_dict:
                 return [
-                    self.convert_err_corr_dict_to_num(unc.err_corr_dict(), var_dims)
+                    self._convert_err_corr_dict_to_num(unc.err_corr_dict(), var_dims)
                     for unc in dsu
                 ]
             else:
@@ -603,7 +634,7 @@ class MeasurementFunctionUtils:
         elif form == "stru":
             if self.use_err_corr_dict:
                 return [
-                    self.convert_err_corr_dict_to_num(unc.err_corr_dict(), var_dims)
+                    self._convert_err_corr_dict_to_num(unc.err_corr_dict(), var_dims)
                     for unc in dsu
                     if unc.is_structured
                 ]
@@ -631,7 +662,7 @@ class MeasurementFunctionUtils:
             try:
                 uvar = "%s_%s" % (form, var)
                 if self.use_err_corr_dict:
-                    return self.convert_err_corr_dict_to_num(
+                    return self._convert_err_corr_dict_to_num(
                         dsu[uvar].err_corr_dict(), var_dims
                     )
                 else:
@@ -644,7 +675,7 @@ class MeasurementFunctionUtils:
                 uvar = keys[uvar_ids]
                 if len(uvar) > 0:
                     if self.use_err_corr_dict:
-                        return self.convert_err_corr_dict_to_num(
+                        return self._convert_err_corr_dict_to_num(
                             dsu[uvar[0]].err_corr_dict(), var_dims
                         )
                     else:
@@ -652,7 +683,7 @@ class MeasurementFunctionUtils:
                 else:
                     return None
 
-    def convert_err_corr_dict_to_num(self, err_corr_dict, var_dims):
+    def _convert_err_corr_dict_to_num(self, err_corr_dict, var_dims):
         """
         Function to convert err_corr dictionary with dimension names as key, to dictionary with dimension indices as keys
 
