@@ -1,6 +1,7 @@
 """
 Tests for mc propagation class
 """
+
 import os.path
 import unittest
 
@@ -8,7 +9,9 @@ import numpy as np
 import numpy.testing as npt
 import xarray as xr
 
-from punpy import MeasurementFunction,MCPropagation
+np.random.seed(2222)
+
+from punpy import MeasurementFunction, MCPropagation
 
 """___Authorship___"""
 __author__ = "Pieter De Vis"
@@ -28,12 +31,12 @@ class HypernetsMF(MeasurementFunction):
         corrected_DN = DN / (
             non_linear[0]
             + non_linear[1] * DN
-            + non_linear[2] * DN ** 2
-            + non_linear[3] * DN ** 3
-            + non_linear[4] * DN ** 4
-            + non_linear[5] * DN ** 5
-            + non_linear[6] * DN ** 6
-            + non_linear[7] * DN ** 7
+            + non_linear[2] * DN**2
+            + non_linear[3] * DN**3
+            + non_linear[4] * DN**4
+            + non_linear[5] * DN**5
+            + non_linear[6] * DN**6
+            + non_linear[7] * DN**7
         )
         if gains.ndim == 1:
             return gains[:, None] * corrected_DN / int_time * 1000
@@ -63,12 +66,25 @@ class IdealGasLaw(MeasurementFunction):
         return (n * temp * 8.134) / pres
 
 
+# Define your measurement function inside a subclass of MeasurementFunction
+class IdealGasLaw_R(MeasurementFunction):
+    @staticmethod
+    def meas_function(pres, temp, n, R):
+        return (n * temp * R) / pres
+
+
 # Define your measurement function inside a subclass of MeasurementFunction
 class IdealGasLaw_2out(MeasurementFunction):
     def meas_function(self, pres, temp, n):
         return (n * temp * 8.134) / pres, pres / temp
 
 
+# Define your measurement function inside a subclass of MeasurementFunction
+class IdealGasLaw_2out_diffdim(MeasurementFunction):
+    def meas_function(self, pres, temp, n):
+        return (n * temp * 8.134) / pres, np.mean(pres / temp, axis=0)
+
+
 dir_path = os.path.dirname(os.path.realpath(__file__))
 ds = xr.open_dataset(os.path.join(dir_path, "digital_effects_table_gaslaw_example.nc"))
 
@@ -114,7 +130,18 @@ class TestMeasurementFunction(unittest.TestCase):
 
         npt.assert_allclose(ds_y_tot["u_tot_volume"].values, u_tot_volume, rtol=0.1)
 
-        prop = MCPropagation(3000, dtype="float32", verbose=False)
+        prop = MCPropagation(1000, dtype="float32", verbose=False, parallel_cores=1)
+
+        gl = IdealGasLaw(
+            prop,
+            ["pressure", "temperature", "n_moles"],
+            yvariable="volume",
+            yunit="m^3",
+            use_err_corr_dict=True,
+        )
+        ds_y_tot = gl.propagate_ds_total(ds)
+
+        prop = MCPropagation(1000, dtype="float32", verbose=False)
         gl = IdealGasLaw(
             prop,
             ["pressure", "temperature", "n_moles"],
@@ -129,10 +156,10 @@ class TestMeasurementFunction(unittest.TestCase):
         )
 
         npt.assert_allclose(ds_y["volume"].values, volume, rtol=0.002)
-        npt.assert_allclose(ds_y["u_ran_volume"].values, u_ran_volume, rtol=0.08)
-        npt.assert_allclose(ds_y["u_sys_volume"].values, u_sys_volume, rtol=0.08)
-        npt.assert_allclose(ds_y["u_str_volume"].values, u_str_volume, rtol=0.08)
-        npt.assert_allclose(ds_y["u_str_volume"].values, u_str_volume, rtol=0.08)
+        npt.assert_allclose(ds_y["u_ran_volume"].values, u_ran_volume, rtol=0.1)
+        npt.assert_allclose(ds_y["u_sys_volume"].values, u_sys_volume, rtol=0.1)
+        npt.assert_allclose(ds_y["u_str_volume"].values, u_str_volume, rtol=0.1)
+        npt.assert_allclose(ds_y["u_str_volume"].values, u_str_volume, rtol=0.1)
         npt.assert_allclose(
             ds_y.unc["volume"].total_unc().values, u_tot_volume, rtol=0.12
         )
@@ -149,8 +176,20 @@ class TestMeasurementFunction(unittest.TestCase):
 
         npt.assert_allclose(ds_y["volume"].values, volume, rtol=0.002)
 
+    def test_gaslaw_scalar(self):
+        prop = MCPropagation(100, dtype="float32", verbose=True, parallel_cores=1)
+
+        gl = IdealGasLaw_R(
+            prop,
+            ["pressure", "temperature", "n_moles", "R"],
+            yvariable="volume",
+            yunit="m^3",
+        )
+        ds_y_tot = gl.propagate_ds_total(ds)
+        npt.assert_allclose(ds_y_tot["volume"].values, volume, rtol=0.03)
+
     def test_gaslaw_2out(self):
-        prop = MCPropagation(1000, dtype="float32", verbose=False)
+        prop = MCPropagation(1000, dtype="float32", verbose=True)
 
         gl = IdealGasLaw_2out(
             prop,
@@ -161,9 +200,73 @@ class TestMeasurementFunction(unittest.TestCase):
         ds_y_tot = gl.propagate_ds_total(ds)
 
         npt.assert_allclose(ds_y_tot["volume"].values, volume, rtol=0.002)
+        npt.assert_equal(ds_y_tot["err_corr_tot_volume"].values.shape, (3600, 3600))
+        npt.assert_equal(ds_y_tot["err_corr_tot_P/T"].values.shape, (3600, 3600))
 
         npt.assert_allclose(ds_y_tot["u_tot_volume"].values, u_tot_volume, rtol=0.1)
 
+    def test_gaslaw_2out_diffdim(self):
+        prop = MCPropagation(100, dtype="float32", verbose=False, parallel_cores=1)
+
+        gl = IdealGasLaw_2out_diffdim(
+            prop,
+            ["pressure", "temperature", "n_moles"],
+            yvariable=["volume", "P/T"],
+            yunit=["m^3", "Pa/K"],
+            ydims=[["x", "y", "time"], ["y", "time"]],
+        )
+        ds_y = gl.propagate_ds(ds)
+
+        npt.assert_allclose(ds_y["volume"].values, volume, rtol=0.002)
+        npt.assert_equal(ds_y["err_corr_str_volume"].values.shape, (3600, 3600))
+        npt.assert_equal(ds_y["err_corr_str_P/T"].values.shape, (180, 180))
+
+        gl = IdealGasLaw_2out_diffdim(
+            prop,
+            ["pressure", "temperature", "n_moles"],
+            yvariable=["volume", "P/T"],
+            yunit=["m^3", "Pa/K"],
+            ydims=[["x", "y", "time"], ["y", "time"]],
+            corr_dims=["x", None],
+            separate_corr_dims=True,
+        )
+        ds_y = gl.propagate_ds(ds)
+
+        npt.assert_allclose(ds_y["volume"].values, volume, rtol=0.002)
+        npt.assert_equal(ds_y["err_corr_str_volume_x"].values.shape, (20, 20))
+
+        gl = IdealGasLaw_2out_diffdim(
+            prop,
+            ["pressure", "temperature", "n_moles"],
+            yvariable=["volume", "P/T"],
+            yunit=["m^3", "Pa/K"],
+            ydims=[["x", "y", "time"], ["y", "time"]],
+            corr_dims=["x"],
+        )
+        ds_y = gl.propagate_ds(ds)
+
+        npt.assert_allclose(ds_y["volume"].values, volume, rtol=0.002)
+        npt.assert_equal(ds_y["err_corr_str_volume_x"].values.shape, (20, 20))
+
+        gl = IdealGasLaw_2out_diffdim(
+            prop,
+            ["pressure", "temperature", "n_moles"],
+            yvariable=["volume", "P/T"],
+            yunit=["m^3", "Pa/K"],
+            ydims=[["x", "y", "time"], ["y", "time"]],
+            corr_dims=["x", "y.time"],
+        )
+        ds_y_tot = gl.propagate_ds_total(ds)
+
+        npt.assert_allclose(ds_y_tot["volume"].values, volume, rtol=0.002)
+        npt.assert_equal(ds_y_tot["err_corr_tot_volume_x"].values.shape, (20, 20))
+        npt.assert_equal(
+            ds_y_tot["err_corr_tot_volume_y.time"].values.shape, (180, 180)
+        )
+        npt.assert_equal(ds_y_tot["err_corr_tot_P/T_y.time"].values.shape, (180, 180))
+
+        npt.assert_allclose(ds_y_tot["u_tot_volume"].values, u_tot_volume, rtol=0.4)
+
     def test_gaslaw_errcorrdict(self):
         prop = MCPropagation(1000, dtype="float32", verbose=False)
 
@@ -193,7 +296,7 @@ class TestMeasurementFunction(unittest.TestCase):
         npt.assert_allclose(ds_y["volume"].values, volume, rtol=0.002)
         npt.assert_allclose(ds_y["u_ran_volume"].values, u_ran_volume, rtol=0.06)
         npt.assert_allclose(ds_y["u_sys_volume"].values, u_sys_volume, rtol=0.06)
-        npt.assert_allclose(ds_y["u_str_volume"].values, u_str_volume, rtol=0.06)
+        npt.assert_allclose(ds_y["u_str_volume"].values, u_str_volume, rtol=0.07)
         npt.assert_allclose(
             ds_y.unc["volume"].total_unc().values, u_tot_volume, rtol=0.06
         )
@@ -246,7 +349,7 @@ class TestMeasurementFunction(unittest.TestCase):
         )
 
         hmf.setup(0.1)
-        y = hmf.run(calib_data, L0data.variables, L0data)
+        y = hmf.run_meas_function(calib_data, L0data.variables, L0data)
         u_y_rand = hmf.propagate_random(L0data, calib_data)
         print(u_y_rand)
         # print(u_y_rand,L0data)
@@ -262,8 +365,8 @@ class TestMeasurementFunction(unittest.TestCase):
             "u_rel_systematic_corr_rad_irr", L0data, calib_data
         )
 
-        u_y_syst = (u_y_syst_indep ** 2 + u_y_syst_corr ** 2) ** 0.5
-        u_y_tot = (u_y_syst_indep ** 2 + u_y_syst_corr ** 2 + u_y_rand ** 2) ** 0.5
+        u_y_syst = (u_y_syst_indep**2 + u_y_syst_corr**2) ** 0.5
+        u_y_tot = (u_y_syst_indep**2 + u_y_syst_corr**2 + u_y_rand**2) ** 0.5
 
         ds_tot = hmf.propagate_ds_total(L0data, calib_data, store_unc_percent=True)
         mask = np.where(
@@ -272,8 +375,8 @@ class TestMeasurementFunction(unittest.TestCase):
         npt.assert_allclose(
             ds_tot["u_rel_tot_irradiance"][mask],
             u_y_tot[mask] / y[mask] * 100,
-            rtol=0.05,
-            atol=0.05,
+            rtol=0.07,
+            atol=0.07,
         )
 
     def test_hypernets_expand(self):
@@ -324,7 +427,7 @@ class TestMeasurementFunction(unittest.TestCase):
             corr_dims=-99,
         )
         hmf.setup(0.1)
-        y = hmf.run(calib_data, L0data)
+        y = hmf.run_meas_function(calib_data, L0data)
         u_y_rand = hmf.propagate_random(L0data, calib_data)
         # print(list(L1data.variables))
         mask = np.where(
@@ -363,8 +466,8 @@ class TestMeasurementFunction(unittest.TestCase):
             "u_rel_systematic_corr_rad_irr", L0data, calib_data, return_corr=False
         )
 
-        u_y_syst = (u_y_syst_indep ** 2 + u_y_syst_corr ** 2) ** 0.5
-        u_y_tot = (u_y_syst_indep ** 2 + u_y_syst_corr ** 2 + u_y_rand ** 2) ** 0.5
+        u_y_syst = (u_y_syst_indep**2 + u_y_syst_corr**2) ** 0.5
+        u_y_tot = (u_y_syst_indep**2 + u_y_syst_corr**2 + u_y_rand**2) ** 0.5
 
         npt.assert_allclose(
             L1data["u_rel_systematic_indep_irradiance"].values[mask],
