@@ -4,6 +4,7 @@ from multiprocessing import Pool
 
 import comet_maths as cm
 import numpy as np
+from inspect import signature
 
 import punpy.utilities.utilities as util
 
@@ -16,16 +17,20 @@ __status__ = "Development"
 
 
 class LPUPropagation:
+    """
+    Class to propagate uncertainties using the Law of Propagation of Uncertainty
+
+    :param parallel_cores: number of CPU to be used in parallel processing
+    :type parallel_cores: int
+    :param Jx_diag: Bool to indicate whether the Jacobian matrix can be described with semi-diagonal elements. With this we mean that the measurand has the same shape as each of the input quantities and the square jacobain between the measurand and each of the input quantities individually, only has diagonal elements. Defaults to False
+    :rtype Jx_diag: bool, optional
+    :param step: Defines the spacing used when calculating the Jacobian with numdifftools
+    :type step: float
+    :param verbose: bool to set if logging info should be printed
+    :type verbose: bool
+    """
+
     def __init__(self, parallel_cores=0, Jx_diag=False, step=None, verbose=False):
-        """
-        Initialise Law of Propagation of Uncertainty Propagator
-
-        :param parallel_cores: number of CPU to be used in parallel processing
-        :type parallel_cores: int
-        :param Jx_diag: Bool to indicate whether the Jacobian matrix can be described with semi-diagonal elements. With this we mean that the measurand has the same shape as each of the input quantities and the square jacobain between the measurand and each of the input quantities individually, only has diagonal elements. Defaults to False
-        :rtype Jx_diag: bool, optional
-        """
-
         self.parallel_cores = parallel_cores
         self.Jx_diag = Jx_diag
         self.step = step
@@ -325,7 +330,7 @@ class LPUPropagation:
             corr_dims,
             fixed_corr,
             Jx_diag,
-        ) = self.perform_checks(
+        ) = self._perform_checks(
             func,
             x,
             u_x,
@@ -375,7 +380,7 @@ class LPUPropagation:
                 for i in range(n_repeats):
                     outs[i] = self.propagate_standard(*inputs[i])
 
-            return self.combine_repeated_outs(
+            return self._combine_repeated_outs(
                 outs,
                 yshape,
                 n_repeats,
@@ -395,10 +400,11 @@ class LPUPropagation:
 
             corrs = np.empty(len(x), dtype=object)
             for i in range(len(x)):
-                if corr_x[i] == "syst":
-                    corrs[i] = np.ones((len(x[i].ravel()), len(x[i].ravel())))
-                elif corr_x[i] == "rand":
-                    corrs[i] = np.eye(len(x[i].ravel()))
+                if isinstance(corr_x[i], str):
+                    if corr_x[i] == "syst":
+                        corrs[i] = np.ones((len(x[i].ravel()), len(x[i].ravel())))
+                    elif corr_x[i] == "rand":
+                        corrs[i] = np.eye(len(x[i].ravel()))
                 elif corr_x[i] is None:
                     corrs[i] = np.zeros((len(x[i].ravel()), len(x[i].ravel())))
                 else:
@@ -418,7 +424,7 @@ class LPUPropagation:
                 return_Jacobian,
             )
 
-    def propagate_flattened_cov(
+    def propagate_cov_flattened(
         self,
         func,
         x,
@@ -477,7 +483,7 @@ class LPUPropagation:
             corr_dims,
             fixed_corr,
             Jx_diag,
-        ) = self.perform_checks(
+        ) = self._perform_checks(
             func,
             x,
             None,
@@ -556,7 +562,7 @@ class LPUPropagation:
                 else:
                     return u_func, corr_ys, corr_out, J
 
-    def perform_checks(
+    def _perform_checks(
         self,
         func,
         x,
@@ -597,9 +603,15 @@ class LPUPropagation:
         xshapes = [xi.shape for xi in x]
         if (xshapes.count(xshapes[0]) == len(xshapes)) and repeat_dims == -99:
             if output_vars == 1:
-                fun = lambda c: func(*c.reshape(len(x), -1))
+                fun = lambda c: (
+                    func(*c) if len(c) == len(xshapes) else func(*c.reshape(len(x), -1))
+                )
             else:
-                fun = lambda c: np.concatenate(func(*c.reshape(len(x), -1)))
+                fun = lambda c: (
+                    np.concatenate(func(*c))
+                    if len(c) == len(xshapes)
+                    else np.concatenate(func(*c.reshape(len(x), -1)))
+                )
         else:
 
             clims = [0]
@@ -677,12 +689,13 @@ class LPUPropagation:
             fixed_corr_var = -99
 
         if fixed_corr_var >= 0 and corr_x is not None:
-            if corr_x[fixed_corr_var] == "rand":
-                fixed_corr = np.eye(len(u_x[fixed_corr_var]))
-            elif corr_x[fixed_corr_var] == "syst":
-                fixed_corr = np.ones(
-                    (len(u_x[fixed_corr_var]), len(u_x[fixed_corr_var]))
-                )
+            if isinstance(corr_x[fixed_corr_var], str):
+                if corr_x[fixed_corr_var] == "rand":
+                    fixed_corr = np.eye(len(u_x[fixed_corr_var]))
+                elif corr_x[fixed_corr_var] == "syst":
+                    fixed_corr = np.ones(
+                        (len(u_x[fixed_corr_var]), len(u_x[fixed_corr_var]))
+                    )
             else:
                 fixed_corr = corr_x[fixed_corr_var]
 
@@ -744,7 +757,7 @@ class LPUPropagation:
             Jx_diag,
         )
 
-    def combine_repeated_outs(
+    def _combine_repeated_outs(
         self,
         outs,
         yshape,
