@@ -18,7 +18,7 @@ __status__ = "Development"
 
 
 def function(x1, x2):
-    return x1 ** 2 - 10 * x2
+    return x1**2 - 10 * x2
 
 
 def Jac_function(x1, x2):
@@ -37,11 +37,11 @@ xs = np.array([x1, x2])
 xerrs = np.array([x1err, x2err])
 
 # below, the higher order Taylor expansion terms have been taken into account, and amount to 2.
-yerr_uncorr = 802 ** 0.5 * np.ones(200)
-yerr_corr = 2 ** 0.5 * np.ones(200)
+yerr_uncorr = 802**0.5 * np.ones(200)
+yerr_corr = 2**0.5 * np.ones(200)
 
 # below, the higher order Taylor expansion terms have not been taken into account
-yerr_uncorr_1order = 800 ** 0.5 * np.ones(200)
+yerr_uncorr_1order = 800**0.5 * np.ones(200)
 yerr_corr_1order = 0 * np.ones(200)
 
 
@@ -57,7 +57,7 @@ x2errb = 2 * np.ones((20, 3))
 xsb = np.array([x1b, x2b])
 xerrsb = np.array([x1errb, x2errb])
 
-yerr_uncorrb = 8 ** 0.5 * np.ones((20, 3))
+yerr_uncorrb = 8**0.5 * np.ones((20, 3))
 yerr_corrb = np.zeros((20, 3))
 
 
@@ -84,8 +84,8 @@ x3errc = np.zeros(200)
 xsc = np.array([x1c, x2c, x3c])
 xerrsc = np.array([x1errc, x2errc, x3errc])
 corr_c = np.array([[1, 0.9999999, 0], [0.99999999, 1.0, 0], [0.0, 0.0, 1.0]])
-yerr_uncorrc = 544 ** 0.5 * np.ones(200)
-yerr_corrc = 1024 ** 0.5 * np.ones(200)
+yerr_uncorrc = 544**0.5 * np.ones(200)
+yerr_corrc = 1024**0.5 * np.ones(200)
 
 
 def functiond(x1, x2):
@@ -103,8 +103,25 @@ corr_d = np.ones(
     (2, 2)
 )  # np.array([[1,0.9999999,0.9999999],[0.99999999,1.,0.99999999],[0.9999999,0.9999999,1.]])
 
-yerr_uncorrd = [8 ** 0.5 * np.ones((5, 3, 2)), 8 ** 0.5 * np.ones((5, 3, 2))]
-yerr_corrd = [np.zeros((5, 3, 2)), 16 ** 0.5 * np.ones((5, 3, 2))]
+yerr_uncorrd = [8**0.5 * np.ones((5, 3, 2)), 8**0.5 * np.ones((5, 3, 2))]
+yerr_corrd = [np.zeros((5, 3, 2)), 16**0.5 * np.ones((5, 3, 2))]
+
+
+x1e = np.array(10.0)
+x2e = np.array(30.0)
+x1erre = np.array(1.0)
+x2erre = np.array(2.0)
+
+xse = np.array([x1e, x2e])
+xerrse = np.array([x1erre, x2erre])
+
+# below, the higher order Taylor expansion terms have been taken into account, and amount to 2.
+yerr_uncorre = 802**0.5 * np.ones(1)
+yerr_corre = 2**0.5 * np.ones(1)
+
+# below, the higher order Taylor expansion terms have not been taken into account
+yerr_uncorr_1ordere = 800**0.5 * np.ones(1)
+yerr_corr_1ordere = 0 * np.ones(1)
 
 
 class TestLPUPropagation(unittest.TestCase):
@@ -148,6 +165,37 @@ class TestLPUPropagation(unittest.TestCase):
         )
         npt.assert_allclose(uf, yerr_corr_1order, atol=0.01)
 
+    def test_propagate_random_0D(self):
+        prop = LPUPropagation()
+        uf, ucorr = prop.propagate_random(function, xse, xerrse, return_corr=True)
+        npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.01)
+        npt.assert_allclose(uf, yerr_uncorr_1ordere, rtol=0.01)
+
+        uf, ucorr = prop.propagate_random(
+            function, xse, xerrse, corr_x=["rand", None], return_corr=True
+        )
+        npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.01)
+        npt.assert_allclose(uf, yerr_uncorr_1ordere, rtol=0.01)
+
+        uf, ucorr = prop.propagate_standard(
+            function, xse, xerrse, ["rand", "rand"], return_corr=True
+        )
+        npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.01)
+        npt.assert_allclose(uf, yerr_uncorr_1ordere, rtol=0.01)
+
+        ucov = cm.convert_corr_to_cov(ucorr, uf)
+        ucorr2 = cm.convert_cov_to_corr(ucov, uf)
+        npt.assert_allclose(ucorr, ucorr2, atol=0.01)
+
+        uf = prop.propagate_random(function, xse, xerrse, corr_between=np.ones((2, 2)))
+        npt.assert_allclose(uf, yerr_corr_1ordere, atol=0.01)
+
+        uf, ucorr = prop.propagate_random(function, xse, xerrse, return_corr=True)
+        npt.assert_allclose(uf, yerr_uncorr_1ordere, rtol=0.01)
+
+        uf = prop.propagate_random(function, xse, xerrse, corr_between=np.ones((2, 2)))
+        npt.assert_allclose(uf, yerr_corr_1ordere, atol=0.01)
+
     def test_propagate_random_2D(self):
         prop = LPUPropagation(parallel_cores=2)
         ufb, ucorrb, jac_x = prop.propagate_random(
@@ -348,7 +396,7 @@ class TestLPUPropagation(unittest.TestCase):
         ufb, ucorrb = prop.propagate_cov(
             functionb, xsb, covb, return_corr=True, Jx_diag=True
         )
-        npt.assert_allclose(ufb, yerr_uncorrb * 2 ** 0.5, rtol=0.01)
+        npt.assert_allclose(ufb, yerr_uncorrb * 2**0.5, rtol=0.01)
 
         covb = [
             cm.convert_corr_to_cov(np.eye(len(xerrb.ravel())), xerrb)
@@ -385,7 +433,7 @@ class TestLPUPropagation(unittest.TestCase):
             for xerrc in xerrsc
         ]
         ufc, ucorrc = prop.propagate_cov(functionc, xsc, covc, return_corr=True, Jx=Jx)
-        npt.assert_allclose(ufc, yerr_uncorrc * 2 ** 0.5, rtol=0.01)
+        npt.assert_allclose(ufc, yerr_uncorrc * 2**0.5, rtol=0.01)
 
         covc = [
             cm.convert_corr_to_cov(np.eye(len(xerrc.ravel())), xerrc)
@@ -520,7 +568,7 @@ class TestLPUPropagation(unittest.TestCase):
         )
         npt.assert_allclose(ucorrd[1], np.ones_like(ucorrd[1]), atol=0.01)
 
-    def test_perform_checks(self):
+    def test__perform_checks(self):
         prop = LPUPropagation()
         corrc = [
             np.ones((len(xerrc[0].ravel()), len(xerrc[0].ravel()))) for xerrc in xerrsc
@@ -530,7 +578,7 @@ class TestLPUPropagation(unittest.TestCase):
             for xerrd in xerrsd
         ]
 
-        out = prop.perform_checks(
+        out = prop._perform_checks(
             functionc,
             xsc,
             xerrsc,
@@ -543,7 +591,7 @@ class TestLPUPropagation(unittest.TestCase):
             param_fixed=None,
         )
 
-        out = prop.perform_checks(
+        out = prop._perform_checks(
             functiond,
             xsd,
             xerrsd,
@@ -556,7 +604,7 @@ class TestLPUPropagation(unittest.TestCase):
             param_fixed=[True, False, False],
         )
         try:
-            out = prop.perform_checks(
+            out = prop._perform_checks(
                 functionc,
                 xsc,
                 xerrsc,
@@ -568,7 +616,7 @@ class TestLPUPropagation(unittest.TestCase):
                 Jx_diag=None,
                 param_fixed=None,
             )
-            out = prop.perform_checks(
+            out = prop._perform_checks(
                 functiond,
                 xsd,
                 xerrsd,
