@@ -9,6 +9,8 @@ import numpy as np
 import numpy.testing as npt
 from punpy.mc.mc_propagation import MCPropagation
 
+np.random.seed(12434)
+
 """___Authorship___"""
 __author__ = "Pieter De Vis"
 __created__ = "14/4/2020"
@@ -18,7 +20,7 @@ __status__ = "Development"
 
 
 def function(x1, x2):
-    return x1 ** 2 - 10 * x2 + 30.0
+    return x1**2 - 10 * x2 + 30.0
 
 
 x1 = np.ones(200) * 10
@@ -30,8 +32,8 @@ xs = [x1, x2]
 xerrs = [x1err, x2err]
 
 # below, the higher order Taylor expansion terms have been taken into account, and amount to 2.
-yerr_uncorr = 802 ** 0.5 * np.ones(200)
-yerr_corr = 2 ** 0.5 * np.ones(200)
+yerr_uncorr = 802**0.5 * np.ones(200)
+yerr_corr = 2**0.5 * np.ones(200)
 
 
 def functionb(x1, x2):
@@ -60,7 +62,7 @@ x2errb = 2 * np.ones((20, 30))
 xsb = np.array([x1b, x2b])
 xerrsb = np.array([x1errb, x2errb])
 
-yerr_uncorrb = 8 ** 0.5 * np.ones((20, 30))
+yerr_uncorrb = 8**0.5 * np.ones((20, 30))
 yerr_corrb = np.zeros((20, 30))
 
 
@@ -79,8 +81,8 @@ x3errc = np.zeros(200)
 xsc = np.array([x1c, x2c, x3c])
 xerrsc = np.array([x1errc, x2errc, x3errc])
 corr_c = np.array([[1, 0.9999999, 0], [0.99999999, 1.0, 0], [0.0, 0.0, 1.0]])
-yerr_uncorrc = 544 ** 0.5 * np.ones(200)
-yerr_corrc = 1024 ** 0.5 * np.ones(200)
+yerr_uncorrc = 544**0.5 * np.ones(200)
+yerr_corrc = 1024**0.5 * np.ones(200)
 
 
 def functiond(x1, x2):
@@ -99,10 +101,10 @@ corr_d = np.ones(
 )  # np.array([[1,0.9999999,0.9999999],[0.99999999,1.,0.99999999],[0.9999999,0.9999999,1.]])
 
 yerr_uncorrd = [
-    np.array(8 ** 0.5 * np.ones((20, 3, 4))),
-    np.array(8 ** 0.5 * np.ones((20, 3, 4))),
+    np.array(8**0.5 * np.ones((20, 3, 4))),
+    np.array(8**0.5 * np.ones((20, 3, 4))),
 ]
-yerr_corrd = [np.zeros((20, 3, 4)), 16 ** 0.5 * np.ones((20, 3, 4))]
+yerr_corrd = [np.zeros((20, 3, 4)), 16**0.5 * np.ones((20, 3, 4))]
 
 
 def functione(x1, x2):
@@ -115,11 +117,16 @@ class TestMCPropagation(unittest.TestCase):
     """
 
     def test_propagate_random_1D(self):
-        prop = MCPropagation(40000, parallel_cores=0)
+        prop = MCPropagation(0, parallel_cores=0)
+        uf, ucorr = prop.propagate_random(function, xs, xerrs, return_corr=True)
+        assert uf is None
+        assert ucorr is None
 
+        prop = MCPropagation(40000, parallel_cores=0)
         uf, ucorr = prop.propagate_random(function, xs, xerrs, return_corr=True)
         npt.assert_allclose(ucorr, np.eye(len(ucorr)), atol=0.06)
         npt.assert_allclose(uf, yerr_uncorr, rtol=0.06)
+
         ucov = cm.convert_corr_to_cov(ucorr, uf)
         ucorr2 = cm.convert_cov_to_corr(ucov, uf)
         npt.assert_allclose(ucorr, ucorr2, atol=0.01)
@@ -131,6 +138,8 @@ class TestMCPropagation(unittest.TestCase):
             function, xs, xerrs, return_corr=True, return_samples=True
         )
 
+        print(xvalues)
+
         npt.assert_allclose(uf, yerr_uncorr, rtol=0.06)
 
         uf, yvalues, xvalues = prop.propagate_random(
@@ -364,8 +373,8 @@ class TestMCPropagation(unittest.TestCase):
         ufc = prop.propagate_systematic(functionc, xsc, xerrsc, corr_between=corr_c)
         npt.assert_allclose(ufc, yerr_corrc, rtol=0.06)
 
-        xsc2 = np.array([x1c, 10.0, x3c])
-        xerrsc2 = np.array([x1errc, 5.0, x3errc])
+        xsc2 = [x1c, 10.0, x3c]
+        xerrsc2 = [x1errc, 5.0, x3errc]
 
         ufc, ucorrc = prop.propagate_systematic(
             functionc, xsc2, xerrsc2, return_corr=True, param_fixed=[False, True, False]
@@ -373,8 +382,8 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ucorrc, np.ones_like(ucorrc), atol=0.06)
         npt.assert_allclose(ufc, yerr_uncorrc, rtol=0.06)
 
-        xsc2 = np.array([x1c, 10.0, x3c])
-        xerrsc2 = np.array([x1errc, 5.0, x3errc])
+        xsc2 = [x1c, 10.0, x3c]
+        xerrsc2 = [x1errc, 5.0, x3errc]
 
         ufc = prop.propagate_systematic(functionc, xsc2, xerrsc2)
         npt.assert_allclose(ufc, yerr_uncorrc, rtol=0.06)
@@ -421,7 +430,7 @@ class TestMCPropagation(unittest.TestCase):
             for xerr in xerrs
         ]
         uf, ucorr = prop.propagate_cov(function, xs, cov, return_corr=True)
-        npt.assert_allclose(uf, yerr_uncorr * 2 ** 0.5, rtol=0.06)
+        npt.assert_allclose(uf, yerr_uncorr * 2**0.5, rtol=0.06)
 
         cov = [
             cm.convert_corr_to_cov(np.eye(len(xerr.ravel())), xerr) for xerr in xerrs
@@ -483,7 +492,7 @@ class TestMCPropagation(unittest.TestCase):
             for xerrb in xerrsb
         ]
         ufb, ucorrb = prop.propagate_cov(functionb, xsb, covb, return_corr=True)
-        npt.assert_allclose(ufb, yerr_uncorrb * 2 ** 0.5, rtol=0.06)
+        npt.assert_allclose(ufb, yerr_uncorrb * 2**0.5, rtol=0.06)
 
         covb = [
             cm.convert_corr_to_cov(np.eye(len(xerrb.ravel())), xerrb)
@@ -513,7 +522,7 @@ class TestMCPropagation(unittest.TestCase):
             for xerrc in xerrsc
         ]
         ufc, ucorrc = prop.propagate_cov(functionc, xsc, covc, return_corr=True)
-        npt.assert_allclose(ufc, yerr_uncorrc * 2 ** 0.5, rtol=0.06)
+        npt.assert_allclose(ufc, yerr_uncorrc * 2**0.5, rtol=0.06)
 
         covc = [
             cm.convert_corr_to_cov(np.eye(len(xerrc.ravel())), xerrc)
@@ -603,6 +612,39 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ucorrb2, np.ones_like(ucorrb2), atol=0.06)
         npt.assert_allclose(ufb, yerr_uncorrb, rtol=0.06)
 
+    def test_separate_corr_dims(self):
+        prop = MCPropagation(20000, parallel_cores=1)
+
+        ufd, ucorrd, corr_out = prop.propagate_systematic(
+            functione,
+            xsd,
+            xerrsd,
+            return_corr=True,
+            corr_dims=[-99, 1],
+            separate_corr_dims=True,
+            output_vars=2,
+            param_fixed=[False, True],
+        )
+        npt.assert_allclose(ucorrd[0], np.ones((240, 240)), atol=0.06)
+        npt.assert_allclose(ucorrd[1], np.ones((4, 4)), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+
+        prop = MCPropagation(20000, parallel_cores=1)
+
+        ufd, ucorrd, corr_out = prop.propagate_systematic(
+            functione,
+            xsd,
+            xerrsd,
+            return_corr=True,
+            corr_dims=[None, 1],
+            separate_corr_dims=True,
+            output_vars=2,
+            param_fixed=[False, True],
+        )
+        npt.assert_equal(ucorrd[0], [None])
+        npt.assert_allclose(ucorrd[1], np.ones((4, 4)), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+
     def test_propagate_syst_corr_3D_2out(self):
         prop = MCPropagation(20000, parallel_cores=1)
 
@@ -721,6 +763,9 @@ class TestMCPropagation(unittest.TestCase):
         MC_x = prop.generate_MC_sample(xsd, xerrsd, corrd)
         MC_y1 = prop.run_samples(functiond, MC_x, output_vars=2, start=0, end=10000)
         MC_y2 = prop.run_samples(functiond, MC_x, output_vars=2, start=10000, end=20000)
+        MC_y3 = prop.run_samples(functiond, MC_x, output_vars=2, start=0, end=10000, return_objects=True)
+        assert isinstance(MC_y3,object)
+        
         MC_y = prop.combine_samples([MC_y1, MC_y2])
 
         ufd, ucorrd, corr_out = prop.process_samples(
@@ -758,7 +803,61 @@ class TestMCPropagation(unittest.TestCase):
         MC_y = prop.combine_samples([MC_y1, MC_y2])
 
         ufd, ucorrd, corr_out = prop.process_samples(
-            MC_x, MC_y, return_corr=True, corr_dims=0, output_vars=2
+            MC_x,
+            MC_y,
+            return_corr=True,
+            corr_dims=[0, 0],
+            output_vars=2,
+            separate_corr_dims=True,
+        )
+
+        npt.assert_allclose(ucorrd[0], np.eye(len(ucorrd[0])), atol=0.06)
+        npt.assert_allclose(ufd[0], yerr_uncorrd[0], rtol=0.06)
+        npt.assert_allclose(ufd[1], yerr_uncorrd[1], rtol=0.06)
+
+    def test_separate_corrdims(self):
+
+        prop = MCPropagation(30000)
+
+        ufd, ucorrd, corr_out = prop.propagate_systematic(
+            functiond,
+            xsd,
+            xerrsd,
+            return_corr=True,
+            corr_dims=[0, 1],
+            output_vars=2,
+            separate_corr_dims=True,
+        )
+        npt.assert_allclose(ucorrd[0], np.ones_like(ucorrd[0]), atol=0.06)
+        npt.assert_allclose(ucorrd[1], np.ones_like(ucorrd[1]), atol=0.06)
+
+        ufd, ucorrd, corr_out = prop.propagate_systematic(
+            functiond,
+            xsd,
+            xerrsd,
+            return_corr=True,
+            corr_dims=[1, 1],
+            output_vars=2,
+            separate_corr_dims=True,
+        )
+        npt.assert_allclose(ucorrd[1], np.ones_like(ucorrd[1]), atol=0.06)
+
+        prop = MCPropagation(20000, parallel_cores=4)
+
+        corrd = [np.eye(len(xerrd[:, 0, 0].ravel())) for xerrd in xerrsd]
+
+        MC_x = prop.generate_MC_sample(xsd, xerrsd, corr_x=["rand", "rand"])
+        MC_y1 = prop.run_samples(functiond, MC_x, output_vars=2, start=0, end=10000)
+        MC_y2 = prop.run_samples(functiond, MC_x, output_vars=2, start=10000, end=20000)
+        MC_y = prop.combine_samples([MC_y1, MC_y2])
+
+        ufd, ucorrd, corr_out = prop.process_samples(
+            MC_x,
+            MC_y,
+            return_corr=True,
+            corr_dims=[0, 0],
+            output_vars=2,
+            separate_corr_dims=True,
         )
 
         npt.assert_allclose(ucorrd[0], np.eye(len(ucorrd[0])), atol=0.06)
@@ -780,7 +879,7 @@ class TestMCPropagation(unittest.TestCase):
         npt.assert_allclose(ucorrb, np.eye(len(ucorrb)), atol=0.06)
         npt.assert_allclose(ufb, yerr_uncorrb, rtol=0.06)
 
-    def test_perform_checks(self):
+    def test__perform_checks(self):
         prop = MCPropagation(20000)
         corrc = [
             np.ones((len(xerrc[0].ravel()), len(xerrc[0].ravel()))) for xerrc in xerrsc
@@ -789,51 +888,55 @@ class TestMCPropagation(unittest.TestCase):
             np.ones((len(xerrd[0].ravel()), len(xerrd[0].ravel()))) for xerrd in xerrsd
         ]
 
-        out = prop.perform_checks(
+        out = prop._perform_checks(
             functionc,
             xsc,
             xerrsc,
             corr_x=corrc,
             repeat_dims=0,
             corr_dims=1,
+            separate_corr_dims=False,
             output_vars=1,
             fixed_corr_var=None,
             param_fixed=None,
             refyvar=0,
         )
 
-        out = prop.perform_checks(
+        out = prop._perform_checks(
             functiond,
             xsd,
             xerrsd,
             corr_x=corrd,
             repeat_dims=[0, 1],
             corr_dims=2,
+            separate_corr_dims=False,
             output_vars=2,
             fixed_corr_var=None,
             param_fixed=[True, False, False],
             refyvar=0,
         )
         try:
-            out = prop.perform_checks(
+            out = prop._perform_checks(
                 functionc,
                 xsc,
                 xerrsc,
                 corr_x=corrc,
                 repeat_dims=0,
                 corr_dims=0,
+                separate_corr_dims=False,
                 output_vars=1,
                 fixed_corr_var=None,
                 param_fixed=None,
                 refyvar=0,
             )
-            out = prop.perform_checks(
+            out = prop._perform_checks(
                 functiond,
                 xsd,
                 xerrsd,
                 corr_x=corrd,
                 repeat_dims=[0, 1],
                 corr_dims=1,
+                separate_corr_dims=False,
                 output_vars=2,
                 fixed_corr_var=None,
                 param_fixed=[True, False, False],
